@@ -5,10 +5,12 @@ import json
 import math
 import os
 import re
-import shutil
 import sys
 import tempfile
+import sysconfig
 import numpy as np
+import svgpathtools
+from itertools import islice
 
 from wand.api import library
 from wand.color import Color
@@ -17,7 +19,14 @@ from wand.image import Image
 import pcbnew
 from lxml import etree
 
+# Give more priority to local modules than installed versions
 PKG_BASE = os.path.dirname(__file__)
+sys.path.insert(0, os.path.dirname(os.path.abspath(PKG_BASE)))
+from pcbdraw import __version__
+
+STYLES_SUBDIR = 'styles'
+FOOTPRINTS_SUBDIR = 'footprints'
+data_path = [PKG_BASE]
 
 default_style = {
     "copper": "#417e5a",
@@ -37,7 +46,7 @@ float_re = r'([-+]?(?:\d+(?:\.\d*)?|\.\d+)(?:[eE][-+]?\d+)?)'
 
 class SvgPathItem:
     def __init__(self, path):
-        path = re.sub(r"([MLA])(\d+)", r"\1 \2", path)
+        path = re.sub(r"([MLA])(-?\d+)", r"\1 \2", path)
         path = re.split("[, ]", path)
         path = list(filter(lambda x: x, path))
         if path[0] != "M":
@@ -189,6 +198,35 @@ def element_position(element, root=None):
 def ki2dmil(val):
     return val // 2540
 
+def dmil2ki(val):
+    return val * 2540
+
+def ki2mm(val):
+    return val / 1000000.0
+
+def mm2ki(val):
+    return val * 1000000
+
+def to_kicad_basic_units(val):
+    """
+    Read string value and return it as KiCAD base units
+    """
+    x = float_re + r'\s*(pt|pc|mm|cm|in)?'
+    value, unit = re.findall(x, val)[0]
+    value = float(value)
+    if unit == "" or unit == "px":
+        return mm2ki(value * 25.4 / 96)
+    if unit == "pt":
+        return mm2ki(value * 25.4 / 72)
+    if unit == "pc":
+        return mm2ki(value * 25.4 / 6)
+    if unit == "mm":
+        return mm2ki(value)
+    if unit == "cm":
+        return mm2ki(value * 10)
+    if unit == "in":
+        return mm2ki(25.4 * value)
+
 def to_user_units(val):
     x = float_re + r'\s*(pt|pc|mm|cm|in)?'
     value, unit = re.findall(x, val)[0]
@@ -273,39 +311,39 @@ def get_board_polygon(svg_elements):
                 s = " M {0} {1} m-{2} 0 a {2} {2} 0 1 0 {3} 0 a {2} {2} 0 1 0 -{3} 0 ".format(
                     att["cx"], att["cy"], att["r"], 2 * float(att["r"]))
                 path += s
-    outline = [elements[0]]
-    elements = elements[1:]
-    while True:
-        size = len(outline)
-        for i, e in enumerate(elements):
-            if SvgPathItem.is_same(outline[0].start, e.end):
-                outline.insert(0, e)
-            elif SvgPathItem.is_same(outline[0].start, e.start):
-                e.flip()
-                outline.insert(0, e)
-            elif SvgPathItem.is_same(outline[-1].end, e.start):
-                outline.append(e)
-            elif SvgPathItem.is_same(outline[-1].end, e.end):
-                e.flip()
-                outline.append(e)
-            else:
-                continue
-            del elements[i]
-            break
-        if size == len(outline):
-            first = True
-            for x in outline:
-                path += x.format(first)
-                first = False
-            if elements:
-                outline = [elements[0]]
-                elements = elements[1:]
-            else:
-                e = etree.Element("path", d=path, style="fill-rule=evenodd;")
-                return e
+    while len(elements) > 0:
+        # Initiate seed for the outline
+        outline = [elements[0]]
+        elements = elements[1:]
+        size = 0
+        # Append new segments to the ends of outline until there is none to append.
+        while size != len(outline):
+            size = len(outline)
+            for i, e in enumerate(elements):
+                if SvgPathItem.is_same(outline[0].start, e.end):
+                    outline.insert(0, e)
+                elif SvgPathItem.is_same(outline[0].start, e.start):
+                    e.flip()
+                    outline.insert(0, e)
+                elif SvgPathItem.is_same(outline[-1].end, e.start):
+                    outline.append(e)
+                elif SvgPathItem.is_same(outline[-1].end, e.end):
+                    e.flip()
+                    outline.append(e)
+                else:
+                    continue
+                del elements[i]
+                break
+        # ...then, append it to path.
+        first = True
+        for x in outline:
+            path += x.format(first)
+            first = False
+    e = etree.Element("path", d=path, style="fill-rule: evenodd;")
+    return e
 
 def process_board_substrate_layer(container, name, source, colors, boardsize):
-    layer = etree.SubElement(container, "g", id="substrate-"+name,
+    layer = etree.SubElement(container, "g", id="substrate-" + name,
         style="fill:{0}; stroke:{0};".format(colors[name]))
     if name == "pads":
         layer.attrib["mask"] = "url(#pads-mask)"
@@ -356,32 +394,31 @@ def get_layers(board, colors, toPlot):
     Plot given layers, process them and return them as <g>
     """
     container = etree.Element('g')
-    tmp = tempfile.mkdtemp()
-    pctl = pcbnew.PLOT_CONTROLLER(board)
-    popt = pctl.GetPlotOptions()
-    popt.SetOutputDirectory(tmp)
-    popt.SetScale(1)
-    popt.SetMirror(False)
-    popt.SetSubtractMaskFromSilk(True)
-    try:
-        popt.SetPlotOutlineMode(False)
-    except:
-        # Method does not exist in older versions of KiCad
-        pass
-    popt.SetTextMode(pcbnew.PLOTTEXTMODE_STROKE)
-    for f, layers, _ in toPlot:
-        pctl.OpenPlotfile(f, pcbnew.PLOT_FORMAT_SVG, f)
-        for l in layers:
-            pctl.SetColorMode(False)
-            pctl.SetLayer(l)
-            pctl.PlotLayer()
-    pctl.ClosePlot()
-    boardsize = board.ComputeBoundingBox()
-    for f, _, process in toPlot:
-        for svg_file in os.listdir(tmp):
-            if svg_file.endswith("-" + f + ".svg"):
-                process(container, f, os.path.join(tmp, svg_file), colors, boardsize)
-    shutil.rmtree(tmp)
+    with tempfile.TemporaryDirectory() as tmp:
+        pctl = pcbnew.PLOT_CONTROLLER(board)
+        popt = pctl.GetPlotOptions()
+        popt.SetOutputDirectory(tmp)
+        popt.SetScale(1)
+        popt.SetMirror(False)
+        popt.SetSubtractMaskFromSilk(True)
+        try:
+            popt.SetPlotOutlineMode(False)
+        except:
+            # Method does not exist in older versions of KiCad
+            pass
+        popt.SetTextMode(pcbnew.PLOTTEXTMODE_STROKE)
+        for f, layers, _ in toPlot:
+            pctl.OpenPlotfile(f, pcbnew.PLOT_FORMAT_SVG, f)
+            for l in layers:
+                pctl.SetColorMode(False)
+                pctl.SetLayer(l)
+                pctl.PlotLayer()
+        pctl.ClosePlot()
+        boardsize = board.ComputeBoundingBox()
+        for f, _, process in toPlot:
+            for svg_file in os.listdir(tmp):
+                if svg_file.endswith("-" + f + ".svg"):
+                    process(container, f, os.path.join(tmp, svg_file), colors, boardsize)
     return container
 
 def get_board_substrate(board, colors, holes, back):
@@ -390,7 +427,7 @@ def get_board_substrate(board, colors, holes, back):
     return SVG g element with the board substrate
     """
     toPlot = []
-    if(back):
+    if back:
         toPlot = [
             ("board", [pcbnew.Edge_Cuts], process_board_substrate_base),
             ("clad", [pcbnew.B_Mask], process_board_substrate_layer),
@@ -410,32 +447,32 @@ def get_board_substrate(board, colors, holes, back):
             ("outline", [pcbnew.Edge_Cuts], process_board_substrate_layer)]
     container = etree.Element('g')
     container.attrib["clip-path"] = "url(#cut-off)"
-    tmp = tempfile.mkdtemp()
-    pctl = pcbnew.PLOT_CONTROLLER(board)
-    popt = pctl.GetPlotOptions()
-    popt.SetOutputDirectory(tmp)
-    popt.SetScale(1)
-    popt.SetMirror(False)
-    popt.SetSubtractMaskFromSilk(True)
-    try:
-        popt.SetPlotOutlineMode(False)
-    except:
-        # Method does not exist in older versions of KiCad
-        pass
-    popt.SetTextMode(pcbnew.PLOTTEXTMODE_STROKE)
-    for f, layers, _ in toPlot:
-        pctl.OpenPlotfile(f, pcbnew.PLOT_FORMAT_SVG, f)
-        for l in layers:
-            pctl.SetColorMode(False)
-            pctl.SetLayer(l)
-            pctl.PlotLayer()
-    pctl.ClosePlot()
-    boardsize = board.ComputeBoundingBox()
-    for f, _, process in toPlot:
-        for svg_file in os.listdir(tmp):
-            if svg_file.endswith("-" + f + ".svg"):
-                process(container, f, os.path.join(tmp, svg_file), colors, boardsize)
-    shutil.rmtree(tmp)
+
+    with tempfile.TemporaryDirectory() as tmp:
+        pctl = pcbnew.PLOT_CONTROLLER(board)
+        popt = pctl.GetPlotOptions()
+        popt.SetOutputDirectory(tmp)
+        popt.SetScale(1)
+        popt.SetMirror(False)
+        popt.SetSubtractMaskFromSilk(True)
+        try:
+            popt.SetPlotOutlineMode(False)
+        except:
+            # Method does not exist in older versions of KiCad
+            pass
+        popt.SetTextMode(pcbnew.PLOTTEXTMODE_STROKE)
+        for f, layers, _ in toPlot:
+            pctl.OpenPlotfile(f, pcbnew.PLOT_FORMAT_SVG, f)
+            for l in layers:
+                pctl.SetColorMode(False)
+                pctl.SetLayer(l)
+                pctl.PlotLayer()
+        pctl.ClosePlot()
+        boardsize = board.ComputeBoundingBox()
+        for f, _, process in toPlot:
+            for svg_file in os.listdir(tmp):
+                if svg_file.endswith("-" + f + ".svg"):
+                    process(container, f, os.path.join(tmp, svg_file), colors, boardsize)
 
     if holes:
         container.append(get_hole_mask(board))
@@ -443,10 +480,8 @@ def get_board_substrate(board, colors, holes, back):
     return container
 
 def walk_components(board, back, export):
-    module = board.GetModules()
-    while True:
-        if not module:
-            return
+    module = board.GetModules().GetFirst()
+    while module:
         # Top is for Eagle boards imported to KiCAD
         if (str(module.GetLayerName()) in ["Back", "B.Cu"] and not  back) or \
            (str(module.GetLayerName()) in ["Top", "F.Cu"]  and      back):
@@ -478,17 +513,16 @@ def get_hole_mask(board):
     bg.attrib["width"] = str(ki2dmil(bb.GetWidth()))
     bg.attrib["height"] = str(ki2dmil(bb.GetHeight()))
 
-    module = board.GetModules()
+    module = board.GetModules().GetFirst()
     while module:
         if module.GetPadCount() == 0:
             module = module.Next()
             continue
-        pad = module.Pads()
         try:
-            pad.GetPosition()
-        except:
-            # Newest nightly renames Pads to PadList
             pad = module.PadsList()
+        except AttributeError:
+            # Older interfase used Pads()
+            pad = module.Pads()
         while pad:
             pos = pad.GetPosition()
             padOrientation = pad.GetOrientation()
@@ -519,7 +553,11 @@ def get_model_file(paths, lib, name, ref, remapping):
     """ Find model file in library considering component remapping """
     for path in paths:
         if ref in remapping:
-            lib, name = tuple(remapping[ref].split(":"))
+            lib, new_name = tuple(remapping[ref].split(":"))
+            if name.endswith(".back"):
+                name = new_name + ".back"
+            else:
+                name = new_name
         f = os.path.join(path, lib, name + ".svg")
         if os.path.isfile(f):
             return f
@@ -531,7 +569,13 @@ def print_component(paths, lib, name, value, ref, pos, remapping={}):
         ref, lib, name, pos[0], pos[1], math.degrees(pos[2]), f if f else "Not found")
     print(msg)
 
-def component_from_library(lib, name, value, ref, pos, comp, highlight, silent):
+def get_svg2dmil_scale(svg):
+    width = ki2dmil(to_kicad_basic_units(svg.attrib["width"]))
+    height = ki2dmil(to_kicad_basic_units(svg.attrib["height"]))
+    x, y, vw, vh = [float(x) for x in svg.attrib["viewBox"].split()]
+    return width / vw, height / vh
+
+def component_from_library(lib, name, value, ref, pos, comp, highlight, silent, no_warn_back):
     if not name:
         return
     if comp["filter"] is not None and ref not in comp["filter"]:
@@ -539,7 +583,8 @@ def component_from_library(lib, name, value, ref, pos, comp, highlight, silent):
     f = get_model_file(comp["libraries"], lib, name, ref, comp["remapping"])
     if not f:
         if not silent:
-            print("Warning: component '{}' for footprint '{}' from library '{}' was not found".format(name, ref, lib))
+            if name[-5:] != '.back' or not no_warn_back:
+                print("Warning: component '{}' for footprint '{}' from library '{}' was not found".format(name, ref, lib))
         if comp["placeholder"]:
             etree.SubElement(comp["container"], "rect", x=str(ki2dmil(pos[0]) - 150), y=str(ki2dmil(pos[1]) - 150),
                              width="300", height="300", style="fill:red;")
@@ -557,33 +602,30 @@ def component_from_library(lib, name, value, ref, pos, comp, highlight, silent):
         origin.getparent().remove(origin)
     else:
         print("Warning: component '{}' from library '{}' has no ORIGIN".format(name, lib))
-    r.attrib["transform"] = "translate({} {}) scale(393.700787402) rotate({}) translate({}, {})".format(
-            ki2dmil(pos[0]), ki2dmil(pos[1]),
-            -math.degrees(pos[2]), -origin_x, -origin_y)
-    if ref in highlight["items"]:
-        if "width" in svg_tree.attrib and "height" in svg_tree.attrib:
-            w = to_user_units(svg_tree.attrib["width"])
-            h = to_user_units(svg_tree.attrib["height"])
-            build_highlight(highlight, w, h, pos, (origin_x, origin_y), ref)
-        elif "viewBox" in svg_tree.attrib:
-            viewbox = re.split(" |,", svg_tree.attrib["viewBox"])
-            w = to_user_units(viewbox[2])
-            h = to_user_units(viewbox[3])
-            build_highlight(highlight, w, h, pos, (origin_x, origin_y), ref)
-        else:
-            print("Warning: component '{}' from library '{}' has no viewBox. Cannot highlight".format(name, lib))
 
-def build_highlight(preset, width, height, pos, origin, ref):
+    svg_scale_x, svg_scale_y = get_svg2dmil_scale(svg_tree)
+    r.attrib["transform"] = \
+        f"translate({ki2dmil(pos[0])} {ki2dmil(pos[1])}) " + \
+        f"scale({svg_scale_x} {svg_scale_y}) " + \
+        f"rotate({-math.degrees(pos[2])}) " + \
+        f"translate({-origin_x}, {-origin_y})"
+    if ref in highlight["items"]:
+        w = ki2dmil(to_kicad_basic_units(svg_tree.attrib["width"]))
+        h = ki2dmil(to_kicad_basic_units(svg_tree.attrib["height"]))
+        build_highlight(highlight, w, h, pos, (origin_x, origin_y), (svg_scale_x, svg_scale_y), ref)
+
+def build_highlight(preset, width, height, pos, origin, scale, ref):
     h = etree.SubElement(preset["container"], "rect")
-    scale = 393.700787402
     h.attrib["style"] = preset["style"]
     h.attrib["x"] = str(-preset["padding"])
     h.attrib["y"] = str(-preset["padding"])
-    h.attrib["width"] = str(width + 2 * preset["padding"])
-    h.attrib["height"] = str(height + 2 * preset["padding"])
-    h.attrib["transform"] = "translate({} {}) scale(393.700787402) rotate({}) translate({}, {})".format(
-        ki2dmil(pos[0]), ki2dmil(pos[1]),
-        -math.degrees(pos[2]), -origin[0], -origin[1])
+    h.attrib["width"] = str(width / scale[0] + 2 * preset["padding"])
+    h.attrib["height"] = str(height / scale[1] + 2 * preset["padding"])
+    h.attrib["transform"] = \
+        f"translate({ki2dmil(pos[0])} {ki2dmil(pos[1])}) " + \
+        f"scale({scale[0]} {scale[1]}) " + \
+        f"rotate({-math.degrees(pos[2])}) " + \
+        f"translate({-origin[0]}, {-origin[1]})"
     h.attrib["id"] = "h_" + ref
 
 def svg_to_bitmap(infile, outfile, dpi=300):
@@ -593,7 +635,6 @@ def svg_to_bitmap(infile, outfile, dpi=300):
                                             background_color.resource)
         image.read(filename=infile, resolution=dpi)
         _, ext = os.path.splitext(outfile)
-        print(ext)
         if ext.lower() == ".png":
             type = "png32"
         elif ext.lower() in [".jpg", ".jpeg"]:
@@ -604,17 +645,36 @@ def svg_to_bitmap(infile, outfile, dpi=300):
         with open(outfile, "wb") as out:
             out.write(bin_blob)
 
+def find_data_file(name, ext, subdir):
+    if os.path.isfile(name):
+        return name
+    # Not a file here, needs extension?
+    ln = len(ext)
+    if name[-ln:] != ext:
+        name += ext
+        if os.path.isfile(name):
+            return name
+    # Try in the data path
+    for p in data_path:
+        fn = os.path.join(p, subdir, name)
+        if os.path.isfile(fn):
+            return fn
+    raise RuntimeError("Missing '" + subdir + "' " + name)
+
 def load_style(style_file):
-    STYLES = os.path.join(PKG_BASE, "styles")
     if style_file.startswith("builtin:"):
+        STYLES = os.path.join(PKG_BASE, "styles")
         style_file = os.path.join(STYLES, style_file[len("builtin:"):])
+    else:
+        style_file = find_data_file(style_file, '.json', STYLES_SUBDIR)
     try:
         with open(style_file, "r") as f:
             style = json.load(f)
     except IOError:
         raise RuntimeError("Cannot open style " + style_file)
     required = set(["copper", "board", "clad", "silk", "pads", "outline",
-        "highlight-style", "highlight-offset", "highlight-on-top"])
+        "vcut", "highlight-style", "highlight-offset", "highlight-on-top",
+        "highlight-padding"])
     missing = required - set(style.keys())
     if missing:
         raise RuntimeError("Missing following keys in style {}: {}"
@@ -636,13 +696,81 @@ def load_remapping(remap_file):
 
 def adjust_lib_path(path):
     if path == "default" or path == "kicad-default":
-        return os.path.join(PKG_BASE, "footprints", "KiCAD-base")
+        return [os.path.join(p, FOOTPRINTS_SUBDIR, "KiCAD-base") for p in data_path]
     if path == "eagle-default":
-        return os.path.join(PKG_BASE, "footprints", "Eagle-export")
-    return path
+        return [os.path.join(p, FOOTPRINTS_SUBDIR, "Eagle-export") for p in data_path]
+    return [path]
+
+def setup_data_path():
+    global data_path
+    share = os.path.join('share', 'pcbdraw')
+    entries = len(data_path)
+    scheme_names = sysconfig.get_scheme_names()
+    if os.name == 'posix':
+        if 'posix_user' in scheme_names:
+            data_path.append(os.path.join(sysconfig.get_path('data', 'posix_user'), share))
+        if 'posix_prefix' in scheme_names:
+            data_path.append(os.path.join(sysconfig.get_path('data', 'posix_prefix'), share))
+    elif os.name == 'nt':
+        if 'nt_user' in scheme_names:
+            data_path.append(os.path.join(sysconfig.get_path('data', 'nt_user'), share))
+        if 'nt' in scheme_names:
+            data_path.append(os.path.join(sysconfig.get_path('data', 'nt'), share))
+    if len(data_path) == entries:
+        data_path.append(os.path.join(sysconfig.get_path('data'), share))
+
+def merge_bbox(left, right):
+    """
+    Merge bounding boxes in format (xmin, xmax, ymin, ymax)
+    """
+    return tuple([
+        f(l, r) for l, r, f in zip(left, right, [min, max, min, max])
+    ])
+
+def shrink_svg(svgfilepath, shrinkBorder):
+    """
+    Shrink the SVG canvas to the size of the drawing
+    """
+    document = svgpathtools.Document(svgfilepath)
+    paths = document.paths()
+    if len(paths) == 0:
+        return
+    bbox = paths[0].bbox()
+    for x in paths:
+        bbox = merge_bbox(bbox, x.bbox())
+    bbox = list(bbox)
+    bbox[0] -= ki2dmil(mm2ki(shrinkBorder))
+    bbox[1] += ki2dmil(mm2ki(shrinkBorder))
+    bbox[2] -= ki2dmil(mm2ki(shrinkBorder))
+    bbox[3] += ki2dmil(mm2ki(shrinkBorder))
+    svg = document.tree
+    root = svg.getroot()
+    root.attrib["viewBox"] = "{} {} {} {}".format(
+        bbox[0], bbox[2],
+        bbox[1] - bbox[0], bbox[3] - bbox[2]
+    )
+    root.attrib["width"] = str(ki2mm(dmil2ki(bbox[1] - bbox[0]))) + "mm"
+    root.attrib["height"] = str(ki2mm(dmil2ki(bbox[3] - bbox[2]))) + "mm"
+    document.save(svgfilepath)
+
+def postprocess_svg(svgfilepath, shrinkBorder):
+    if shrinkBorder is not None:
+        shrink_svg(svgfilepath, shrinkBorder)
+    # TBA: Add compression and optimization
 
 def main():
-    parser = argparse.ArgumentParser()
+    setup_data_path()
+    epilog = "Searching for styles on: "
+    c = len(data_path)
+    for i, path in enumerate(data_path):
+        epilog += "'"+os.path.join(path, 'styles')+"'"
+        if i == c-2:
+            epilog += " and "
+        elif i != c-1:
+            epilog += ", "
+
+    parser = argparse.ArgumentParser(epilog=epilog)
+    parser.add_argument("--version", action="version", version=f"PcbDraw {__version__}")
     parser.add_argument("-s", "--style", help="JSON file with board style")
     parser.add_argument("board", help=".kicad_pcb file to draw")
     parser.add_argument("output", help="destination for final SVG or PNG file")
@@ -661,9 +789,14 @@ def main():
     parser.add_argument("-v", "--vcuts", action="store_true", help="Render V-CUTS on the Cmts.User layer")
     parser.add_argument("--silent", action="store_true", help="Silent warning messages about missing footprints")
     parser.add_argument("--dpi", help="DPI for bitmap output", type=int, default=300)
+    parser.add_argument("--no-warn-back", action="store_true", help="Don't show warnings about back footprints")
+    parser.add_argument("--shrink", type=float, help="Shrink the canvas size to the size of the board. Specify border in millimeters")
 
     args = parser.parse_args()
-    args.libs = [adjust_lib_path(path) for path in args.libs.split(',')]
+    libs = []
+    for path in args.libs.split(','):
+        libs.extend(adjust_lib_path(path))
+    args.libs = libs
     args.highlight = args.highlight.split(',') if args.highlight is not None else []
     args.filter = args.filter.split(',') if args.filter is not None else None
 
@@ -695,14 +828,20 @@ def main():
 
     bb = board.ComputeBoundingBox()
     transform_string = ""
-    if(args.back ^ args.mirror):
-        transform_string = "translate({}, {}) scale(-1,1)".format(ki2dmil(bb.GetX()+bb.GetWidth()), ki2dmil(-bb.GetY()))
-    else:
-        transform_string = "translate({}, {})".format(ki2dmil(-bb.GetX()), ki2dmil(-bb.GetY()))
+    # Let me briefly explain what's going on. KiCAD outputs SVG in user units,
+    # where 1 unit is 1/10 of an inch (don't ask me why). So to make our life
+    # easy, we respect it and make our document also in decimils. Therefore we
+    # specify the outer dimensions in millimeters and specify the board area.
     document = empty_svg(
-        width="{}cm".format(bb.GetWidth()/10000000.0),
-        height="{}cm".format(bb.GetHeight()/10000000.0),
-        viewBox="0 0 {} {}".format(ki2dmil(bb.GetWidth()), ki2dmil(bb.GetHeight())))
+            width=f"{ki2mm(bb.GetWidth())}mm",
+            height=f"{ki2mm(bb.GetHeight())}mm",
+            viewBox=f"{ki2dmil(bb.GetX())} {ki2dmil(bb.GetY())} {ki2dmil(bb.GetWidth())} {ki2dmil(bb.GetHeight())}")
+    if(args.back ^ args.mirror):
+        transform_string = "scale(-1,1)"
+        document = empty_svg(
+            width=f"{ki2mm(bb.GetWidth())}mm",
+            height=f"{ki2mm(bb.GetHeight())}mm",
+            viewBox=f"{ki2dmil(-bb.GetWidth() - bb.GetX())} {ki2dmil(bb.GetY())} {ki2dmil(bb.GetWidth())} {ki2dmil(bb.GetHeight())}")
 
     board_cont = etree.SubElement(document.getroot(), "g", transform=transform_string)
     if style["highlight-on-top"]:
@@ -736,19 +875,22 @@ def main():
         board_cont.append(get_layers(board, style, [("vcut", [pcbnew.Cmts_User], process_board_substrate_layer)]))
 
     walk_components(board, args.back, lambda lib, name, val, ref, pos:
-        component_from_library(lib, name, val, ref, pos, components, highlight, args.silent))
+        component_from_library(lib, name, val, ref, pos, components, highlight, args.silent, args.no_warn_back))
 
     # make another pass for search, and if found, render the back side of the component
     # the function will search for file with extension ".back.svg"
     walk_components(board, not args.back, lambda lib, name, val, ref, pos:
-        component_from_library(lib, name+".back", val, ref, pos, components, highlight, args.silent))
+        component_from_library(lib, name+".back", val, ref, pos, components, highlight, args.silent, args.no_warn_back))
 
     if args.output.endswith(".svg") or args.output.endswith(".SVG"):
         document.write(args.output)
+        postprocess_svg(args.output, args.shrink)
     else:
         with tempfile.NamedTemporaryFile(suffix=".svg") as tmp_f:
             document.write(tmp_f)
             tmp_f.flush()
+            postprocess_svg(args.output, args.shrink)
+            tmp_f.flush()
             svg_to_bitmap(tmp_f.name, args.output, dpi=args.dpi)
 
 if __name__ == '__main__':
